Attribute VB_Name = "modMain"
Option Explicit
Option Compare Binary
Option Base 0


Private Enum enumGFXColumn
    GFX_COMMAND
    GFX_ADDRESS
    GFX_SIZE
    GFX_PLANES
    GFX_MODE
    GFX_MASK
    GFX_COUNT
    GFX_FILTER
    GFX_OUTPUT
    GFX_DESCRIPTION
    
    GFX_MAX
End Enum


' Size of a single tile
Public Const TILESIZE As Long = 32

' Data pointer offset
Public Const DATAOFFSET As Long = &H20700

' Blockmap block size
Public Const BMAP_WIDTH As Long = 32
Public Const BMAP_HEIGHT As Long = 32

' Application
Public APP_PATH As String
Public APP_VERSION As String

' Common
Public FS As New FileSystemObject
Public Utils As New clsUtils
Public rFont As clsFont

' Graphics
Public Palettes As Dictionary
Public Graphics As Dictionary
Public Checkers As clsSurface


Public Sub initGraphics()
    
    Dim X As Long, Y As Long
    Dim V As Byte
    
    Const CHECKER_SIZE As Long = 4
    
    
    ' Checkers pattern
    Set Checkers = New clsSurface
    Checkers.Create 128, 128
    For X = 0 To Checkers.Width / CHECKER_SIZE
        For Y = 0 To Checkers.Height / CHECKER_SIZE
            V = (((X + Y) Mod 2) * 63) + 31
            renderBoxFill Checkers.ptrSurface, X * CHECKER_SIZE, Y * CHECKER_SIZE, CHECKER_SIZE, CHECKER_SIZE, createBGRA(V, 0, V, 255), BLEND_SOLID
        Next Y
    Next X

End Sub


Public Sub loadGraphics()

    Dim openFile As Long
    
    Dim Data As String
    Dim Cmd() As String
    Dim Bits As clsBitStream
    
    Dim Pal As clsPalette
    
    Dim Count As Long
    Dim BP As clsBitplane
    Dim BPMask As clsBitplane
    Dim Surf As clsSurface
    Dim Img() As String
    
    Dim Graphic As clsSurfaceList
    
    
    Set Palettes = New Dictionary
    Set Graphics = New Dictionary
    
    openFile = FreeFile
    Open APP_PATH & "graphics.csv" For Input As openFile
    
        ' Skip first line (table header)
        Line Input #openFile, Data
    
        Do
        
            Line Input #openFile, Data
            Cmd = Split(Data, ";")
            
            If UBound(Cmd) > 0 Then
                
                Select Case LCase$(Cmd(GFX_COMMAND))
                
                    ' Empty line
                    Case ""
                
                    ' Open file
                    Case "file"
                        Set Bits = New clsBitStream
                        If Not Bits.fileOpen(APP_PATH & "gamedata\" & Cmd(GFX_ADDRESS)) Then
                            MsgBox "Could not open file " & Cmd(GFX_ADDRESS), vbExclamation Or vbOKOnly
                            Exit Do
                        End If
                        
                    ' Read palette
                    Case "pal"
                        Set Pal = New clsPalette
                        Palettes.Add Cmd(GFX_PLANES), Pal
                        
                        Bits.seekTo CLng(Cmd(GFX_ADDRESS))
                        Pal.loadFrom Bits, CLng(Cmd(GFX_SIZE))
                        
                    ' Set palette
                    Case "setpal"
                        If Not Palettes.Exists(Cmd(GFX_ADDRESS)) Then
                            MsgBox "No palette named " & Cmd(GFX_ADDRESS), vbExclamation Or vbOKOnly
                            Exit Do
                        End If
                        Set Pal = Palettes.Item(Cmd(GFX_ADDRESS))
                    
                    ' Read sprite
                    Case "spr"
                        If Pal Is Nothing Then
                            MsgBox "No palette loaded for bitplane at " & Cmd(GFX_ADDRESS), vbExclamation Or vbOKOnly
                            Exit Do
                        End If
                        
                        Bits.seekTo CLng(Cmd(GFX_ADDRESS))
                        Img = Split(Cmd(GFX_SIZE), "x")
                        
                        Set BP = New clsBitplane
                        For Count = 1 To CLng(Cmd(GFX_COUNT))
                        
                            ' Bitplane format
                            Select Case UCase$(Cmd(GFX_MODE))
                                Case "AM"
                                    If Not BP.loadChunky(Bits, Val(Img(0)), Val(Img(1)), Val(Cmd(GFX_PLANES))) Then
                                        MsgBox "Unable to load AM bitplane at " & Cmd(GFX_ADDRESS), vbExclamation Or vbOKOnly
                                        Exit Do
                                    End If
                                    
                                Case "ST"
                                    If Not BP.loadPlanar(Bits, Val(Img(0)), Val(Img(1)), Val(Cmd(GFX_PLANES))) Then
                                        MsgBox "Unable to load ST bitplane at " & Cmd(GFX_ADDRESS), vbExclamation Or vbOKOnly
                                        Exit Do
                                    End If
                                    
                                Case "AS"
                                    If Not BP.loadAmigaSprite(Bits, Val(Img(0)), Val(Img(1))) Then
                                        MsgBox "Unable to load AS bitplane at " & Cmd(GFX_ADDRESS), vbExclamation Or vbOKOnly
                                        Exit Do
                                    End If
                                    
                                Case Else
                                    MsgBox "Unknown bitplane format " & Cmd(GFX_MODE), vbExclamation Or vbOKOnly
                                    Exit Do
                            End Select
                            
                            
                            ' Mask from color index
                            If IsNumeric(Cmd(GFX_MASK)) Then
                                Set Surf = BP.toSurface(Nothing, Pal.ptrPal, CLng(Cmd(GFX_MASK)), MASKMODE_INDEX)
                            
                            ' No mask
                            ElseIf Cmd(GFX_MASK) = vbNullString Then
                                Set Surf = BP.toSurface(Nothing, Pal.ptrPal, 0, MASKMODE_NONE)
                                
                            ' Mask
                            ElseIf Cmd(GFX_MASK) = "X" Then
                                Set BPMask = New clsBitplane
                                
                                ' Bitplane from ST graphic
                                If Cmd(GFX_MODE) = "ST" Then
                                    If Not BPMask.loadPlanar(Bits, CLng(Img(0)), CLng(Img(1)), Cmd(GFX_PLANES)) Then
                                        MsgBox "Unable to load bitplane mask at " & Cmd(GFX_ADDRESS), vbExclamation Or vbOKOnly
                                        Exit Do
                                    End If
                                
                                ' Bitplane from 1 plane
                                Else
                                    If Not BPMask.loadChunky(Bits, CLng(Img(0)), CLng(Img(1)), 1) Then
                                        MsgBox "Unable to load bitplane mask at " & Cmd(GFX_ADDRESS), vbExclamation Or vbOKOnly
                                        Exit Do
                                    End If
                                End If
                                
                                Set Surf = BP.toSurface(BPMask, Pal.ptrPal, 0, MASKMODE_MASK)
                                
                            Else
                                MsgBox "Unknown mask type " & Cmd(GFX_MASK), vbExclamation Or vbOKOnly
                                Exit Do
                                
                            End If
                            
                            ' Filter
                            If Cmd(GFX_FILTER) <> vbNullString Then
                                Select Case Cmd(GFX_FILTER)
                                    
                                    ' Flip vertically
                                    Case "flipy"
                                        Set Surf = Surf.flipY
                                        
                                    ' Unknown
                                    Case Else
                                        MsgBox "Unknown filter type " & Cmd(GFX_FILTER)
                                        
                                End Select
                            End If
                            
                            ' Add to graphics list
                            If Graphics.Exists(Cmd(GFX_OUTPUT)) Then
                                Set Graphic = Graphics(Cmd(GFX_OUTPUT))
                            Else
                                Set Graphic = New clsSurfaceList
                                Graphic.Name = Cmd(GFX_OUTPUT)
                                Graphics.Add Cmd(GFX_OUTPUT), Graphic
                            End If
                            
                            ' Add surface to graphic
                            Graphic.Add Surf
                            
                        Next Count
                        
                    ' Unknown cmd
                    Case Else
                        MsgBox "Unknown command " & Cmd(GFX_COMMAND), vbExclamation Or vbOKOnly
                        Exit Do
                        
                End Select
                
            End If
        
        Loop Until EOF(openFile)
    
    Close openFile

End Sub


Public Sub Main()
    
    APP_PATH = App.Path
    If Right$(APP_PATH, 1) <> "\" Then APP_PATH = APP_PATH & "\"
    
    APP_VERSION = "Version " & App.Major & "." & App.Minor & " build " & App.Revision
    
    renderInit
    
    initGraphics
    loadGraphics
    
    Set rFont = New clsFont
    If Not rFont.loadFromFile(APP_PATH & "fonts\zepto.png") Then
        MsgBox "Unable to load font.", vbExclamation Or vbOKOnly
    End If

    Load frmAbout
    Load frmImagePeek
    Load frmGraphics
    Load frmMain
    
    Utils.applyDefaultSystemFont
    
    frmMain.Show
    
End Sub


Public Sub Quit()

    Unload frmAbout
    Unload frmImagePeek
    Unload frmGraphics
    Unload frmMain
    
    Set rFont = Nothing

    End

End Sub
